unit PaddleProcessing;

interface

Uses
    System.SysUtils,
    Classes,
    IdMessage,
    IdBaseComponent,
    IdComponent,
    IdTCPConnection,
    IdTCPClient,
    IdHTTP,
    IdIOHandler,
    IdIOHandlerSocket,
    IdIOHandlerStack,
    IdSSL,
    IdSSLOpenSSL;

Const
    BASEURL_PADDLE = 'https://api.paddle.com/'; //'https://sandbox-api.paddle.com/';

type
    TPaddleNotificationInfo = record
        IdMessage: TIdMessage;
        ProductName: String;
        CustomerName: String;
        CustomerID: String;
        CustomerEmail: String;
        CustomerInfo: String;
    end;

    function PaddleProcessNotificationEmail(var PaddleNotificationInfo: TPaddleNotificationInfo): Boolean;
    function PaddleGetCustomerData(ID: String): String;
    function PaddleExtractCustomerInfo(var PaddleNotificationInfo: TPaddleNotificationInfo): Boolean;

var
    SSLAvailable: Boolean;
    IdHTTPPaddle: TIdHTTP;

implementation

Uses
    IdStrings,
    System.JSON;

function PaddleProcessNotificationEmail(var PaddleNotificationInfo: TPaddleNotificationInfo): Boolean;
var
    Text: TStringList;
    ProductName, CustomerName, CustomerID: String;
    ProductsPos, ProductPos, CustomerNamePos, CustomerIDPos: Integer;
begin
    Result := False;
    if NOT SSLAvailable then begin
        raise Exception.Create('Error: no SSLAvailable');
    end;
    Text := TStringList.Create;
    try
        Text.Text := PaddleNotificationInfo.IdMessage.Body.Text;

        //Text.Text := StrHTMLDecode(Text.Text);
        Text.Text := StringReplace(Text.Text, '&amp;', '&', [rfReplaceAll]);
        Text.Text := StringReplace(Text.Text, '&apos;', '''', [rfReplaceAll]);

        //* Product name
        ProductsPos := Pos('line_items', Text.Text);
        if ProductsPos <= 0 then begin
            Exit;
        end;
        ProductPos := Pos('name', Text.Text, ProductsPos);
        if ProductPos <= 0 then begin
            Exit;
        end;
        ProductName := Copy(Text.Text, ProductPos + 18, Pos(',', Text.Text, ProductPos) - (ProductPos + 18));
        ProductName := StringReplace(ProductName, Char($D) + Char($A), '', [rfReplaceAll]);
        ProductName := StringReplace(ProductName, '=', '', [rfReplaceAll]);
        ProductName := StringReplace(ProductName, '&quot;', '', [rfReplaceAll]);
        //* Card holder name
        CustomerNamePos := Pos('cardholder_name', Text.Text);
        if CustomerNamePos > 0 then begin
            CustomerName := Copy(Text.Text, CustomerNamePos + 29, Pos(',', Text.Text, CustomerNamePos) - (CustomerNamePos + 29));
            CustomerName := StringReplace(CustomerName, Char($D) + Char($A), '', [rfReplaceAll]);
            CustomerName := StringReplace(CustomerName, '=', '', [rfReplaceAll]);
            CustomerName := StringReplace(CustomerName, '&quot;', '', [rfReplaceAll]);
            CustomerName := StringReplace(CustomerName, '}', '', [rfReplaceAll]);
            CustomerName := Trim(CustomerName);
        end;
        //* Customer ID
        CustomerIDPos := Pos('ctm_', Text.Text);
        if CustomerIDPos <= 0 then begin
            Exit;
        end;
        CustomerID := Copy(Text.Text, CustomerIDPos, 30);
        //* Return info
        PaddleNotificationInfo.ProductName := ProductName;
        PaddleNotificationInfo.CustomerName := CustomerName;
        PaddleNotificationInfo.CustomerID := CustomerID;
        PaddleNotificationInfo.CustomerEmail := '';
        PaddleNotificationInfo.CustomerInfo := PaddleGetCustomerData(CustomerID);
        PaddleExtractCustomerInfo(PaddleNotificationInfo);
        if PaddleNotificationInfo.CustomerName = '' then begin
            PaddleNotificationInfo.CustomerName := Copy(PaddleNotificationInfo.CustomerEmail, 1, Pos('@', PaddleNotificationInfo.CustomerEmail) - 1);
        end;
        //PaddleExtractCustomerInfo(PaddleNotificationInfo);
        Result := True;
    finally
        FreeAndNil(Text);
    end;
end;


function PaddleGetCustomerData(ID: String): String;
begin
    //IdHTTP1.Request.CustomHeaders.Values['Authorization'] := 'Bearer ' + '18923625bc887a85edfecdc029b66a90ed9fe7d922abef9e57';
    //IdHTTP1.Request.CustomHeaders.FoldLines := True;
    //IdHTTPPaddle.Request.CustomHeaders.Add('Authorization: Bearer ' + APIKeyPaddle);
    //IdHTTP1.Request.CustomHeaders.Add('Paddle-Version: 1');
    Result := IdHTTPPaddle.Get(BASEURL_PADDLE + 'customers/' + ID);
    //Result := IdHTTP1.Get('https://sandbox-api.paddle.com/event-types');
end;

function PaddleExtractCustomerInfo(var PaddleNotificationInfo: TPaddleNotificationInfo): Boolean;
var
    JSONData: TJSONObject;
    ExtractedName: String;
begin
    JSONData := TJSONObject.ParseJSONValue(PaddleNotificationInfo.CustomerInfo) as TJSONObject;
    try
        JSONData.TryGetValue<String>('data.name', ExtractedName);
        if ExtractedName <> '' then begin
            PaddleNotificationInfo.CustomerName := ExtractedName;
        end;
        JSONData.TryGetValue<String>('data.email', PaddleNotificationInfo.CustomerEmail);
        Result := True;
    finally
        FreeAndNil(JSONData);
    end;
end;

Initialization

    SSLAvailable := LoadOpenSSLLibrary;
    IdHTTPPaddle := TIdHTTP.Create(nil);
    if SSLAvailable then begin
        IdHTTPPaddle.IOHandler := TIdSSLIOHandlerSocketOpenSSL.Create;
        (IdHTTPPaddle.IOHandler as TIdSSLIOHandlerSocketOpenSSL).SSLOptions.SSLVersions := [sslvTLSv1, sslvTLSv1_1, sslvTLSv1_2, sslvSSLv2, sslvSSLv3];
    end;
    //IdHTTPPaddle.Request.CustomHeaders.Add('Authorization: Bearer ' + APIKeyPaddle);

Finalization

    FreeAndNil(IdHTTPPaddle);

end.
